@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model;
-$Config::Model::VERSION = '2.096';
+$Config::Model::VERSION = '2.098';
 use strict ;
 use warnings;
 use 5.10.1;
@@ -23,7 +23,7 @@ use Data::Dumper ();
 use Log::Log4perl 1.11 qw(get_logger :levels);
 use Config::Model::Instance;
 use Hash::Merge 0.12 qw/merge/;
-use File::Path qw/make_path/;
+use Path::Tiny 0.053;
 
 use Cwd;
 use Config::Model::Lister;
@@ -35,6 +35,7 @@ our @EXPORT_OK = qw/cme/;
 use vars qw(@status @level %default_property);
 
 my $legacy_logger = get_logger("Model::Legacy") ;
+my $loader_logger = get_logger("Model::Loader") ;
 my $logger = get_logger("Model") ;
 
 # used to keep one Config::Model object to simplify programs based on
@@ -53,7 +54,17 @@ enum LegacyTreament => qw/die warn ignore/;
 has skip_include => ( isa => 'Bool',           is => 'ro', default => 0 );
 has model_dir    => ( isa => 'Str',            is => 'ro', default => 'Config/Model/models' );
 has legacy       => ( isa => 'LegacyTreament', is => 'ro', default => 'warn' );
-has instances => ( isa => 'HashRef[Config::Model::Instance]', is => 'ro', default => sub { {} } );
+has instances => (
+    isa => 'HashRef[Config::Model::Instance]',
+    is => 'ro',
+    default => sub { {} },
+    traits => ['Hash'],
+    handles => {
+        store_instance => 'set',
+        get_instance   => 'get',
+        has_instance   => 'defined',
+    },
+);
 
 # Config::Model stores 3 versions of each model
 
@@ -116,6 +127,20 @@ has models => (
     },
 );
 
+# model snippet may be loaded when the target class is not available
+# so they must be stored before being used.
+has model_snippets => (
+    isa     => 'ArrayRef',
+    is      => 'ro',
+    default => sub { [] },
+    traits  => ['Array'],
+    handles => {
+        add_snippet => 'push',
+        all_snippets => 'elements',
+    },
+);
+
+
 has skip_inheritance => (
     isa     => 'Bool',
     is      => 'ro',
@@ -200,10 +225,10 @@ sub _tweak_instance_args {
         $application .= " in " . cwd;
     }
     $args->{name}
-        =  delete $args->{instance_name}
-        || delete $args->{name}
-        || $app_name
-        || 'default';
+        =  delete $args->{instance_name} # backward compat with test
+        || delete $args->{name}          # preferred parameter
+        || $app_name                     # fallback in most cases
+        || 'default';                    # fallback mostly in tests
 }
 
 sub cme {
@@ -215,7 +240,6 @@ sub cme {
     # model_storage is used to keep Config::Model object alive
     $model_storage //= Config::Model->new(%$m_args);
 
-    my $app = $args{application};
     return $model_storage->instance(@_);
 }
 
@@ -223,18 +247,16 @@ sub instance {
     my $self = shift;
     my %args = @_ == 1 ? ( application => $_[0]) : @_ ;
 
+    # also creates a default name
     _tweak_instance_args(\%args);
 
+    if ( $args{name} and $self->has_instance($args{name}) ) {
+        return $self->get_instance($args{name});
+    }
+
     croak "Model: can't create instance without application or root_class_name "
         unless $args{root_class_name};
 
-    my $instance_name = $args{name};
-    # could add more syntactic suger with 'hash' trait
-    # see Moose::Meta::Attribute::Native
-    if ( defined $self->instances->{$instance_name} ) {
-        return $self->instances->{$instance_name};
-    }
-
     if ( defined $args{model_file} ) {
         my $file = delete $args{model_file};
         $self->load( $args{root_class_name}, $file );
@@ -245,7 +267,7 @@ sub instance {
         %args    # for optional parameters like *directory
     );
 
-    $self->instances->{$instance_name} = $i;
+    $self->store_instance($args{name}, $i);
     return $i;
 }
 
@@ -1170,12 +1192,14 @@ sub load {
     my $model_name = shift;    # model name like Foo::Bar
     my $load_file  = shift;    # model file (override model name), used for tests
 
+    $loader_logger->debug("called on model $model_name");
+
     my $load_path = $model_name;
     $load_path =~ s/::/\//g;
 
     $load_file ||= $self->model_dir . '/' . $load_path . '.pl';
 
-    get_logger("Model::Loader")->debug("model $model_name from file $load_file");
+    $loader_logger->debug("model $model_name from file $load_file");
 
     # no special treatment, returns an array
     my %models_by_name;
@@ -1183,24 +1207,27 @@ sub load {
     $self->store_raw_model( $model_name, dclone( \%models_by_name ) );
     foreach my $name ( keys %models_by_name ) {
         my $data = $self->normalize_class_parameters( $name, $models_by_name{$name} );
-        get_logger("Model::Loader")->debug("Store normalized model $name");
+        $loader_logger->debug("Store normalized model $name");
         $self->store_normalized_model( $name, $data );
     }
 
     # look for additional model information
     my %model_graft_by_name;
     my %done;  # avoid loading twice the same snippet (where system version may clobber dev version)
-    foreach my $inc (@INC,'.') {
+    foreach my $inc_str (@INC,'.') {
         foreach my $name ( keys %models_by_name ) {
             my $snippet_path = $name;
             $snippet_path =~ s/::/\//g;
-            my $snippet_dir = "$inc/" . $self->model_dir . '/' . $snippet_path . '.d';
-            get_logger("Model::Loader")->trace("looking for snippet in $snippet_dir");
-            if ( -d $snippet_dir ) {
-                foreach my $snippet_file ( glob("$snippet_dir/*.pl") ) {
+            my $snippet_dir = path($inc_str)->child($self->model_dir)->child($snippet_path . '.d');
+            $loader_logger->trace("looking for snippet in $snippet_dir");
+            if ( $snippet_dir->is_dir ) {
+                my $iter = $snippet_dir->iterator({ recurse => 1 });
+
+                while ( my $snippet_file = $iter->() ) {
+                    next unless $snippet_file =~ /\.pl$/;
                     my $done_key = $name . ':' . $snippet_file;
                     next if $done{$done_key};
-                    get_logger("Model::Loader")->info("Found snippet $snippet_file");
+                    $loader_logger->info("Found snippet $snippet_file");
                     $self->_load_model_in_hash( \%model_graft_by_name, $snippet_file );
                     $done{$done_key} = 1;
                 }
@@ -1208,9 +1235,18 @@ sub load {
         }
     }
 
-    foreach my $class_to_merge ( keys %model_graft_by_name ) {
-        my $data = $model_graft_by_name{$class_to_merge};
-        $self->augment_config_class_really( $class_to_merge, $data );
+    # store snippet. May be used later
+    foreach my $name (keys %model_graft_by_name) {
+        # store snippet for later usage
+        $loader_logger->trace("storing snippet for model $name");
+        $self->add_snippet($model_graft_by_name{$name});
+    }
+
+    # check if a snippet is available for this class
+    foreach my $snippet ( $self->all_snippets ) {
+        my $class_to_merge = $snippet->{name};
+        next unless $models_by_name{$class_to_merge};
+        $self->augment_config_class_really( $class_to_merge, $snippet );
     }
 
     # return the list of classes found in $load_file. Respecting the order of the class
@@ -1249,7 +1285,7 @@ sub _load_model_in_hash {
 sub _do_model_file {
     my ( $self, $load_file ) = @_;
 
-    get_logger("Model::Loader")->info("load model $load_file");
+    $loader_logger->info("load model $load_file");
 
     my $err_msg = '';
     $load_file = "./$load_file" if $load_file !~ m!^/! and -e $load_file ;
@@ -1288,7 +1324,7 @@ sub augment_config_class_really {
     my $orig_model = $self->normalized_model($config_class_name);
     croak "unknown class to augment: $config_class_name" unless defined $orig_model;
 
-    my $model_addendum = $self->normalize_class_parameters( $config_class_name, $augment_data );
+    my $model_addendum = $self->normalize_class_parameters( $config_class_name, dclone($augment_data) );
 
     my $merge = Hash::Merge->new('RIGHT_PRECEDENT');
     my $new_model = $merge->merge( $orig_model, $model_addendum );
@@ -1317,7 +1353,7 @@ sub get_model {
         unless $self->normalized_model_exists($config_class_name);
 
     if ( not $self->model_defined($config_class_name) ) {
-        get_logger("Model::Loader")->debug("creating model $config_class_name");
+        $loader_logger->debug("creating model $config_class_name");
 
         my $model = $self->merge_included_class($config_class_name);
         $self->_store_model( $config_class_name, $model );
@@ -1509,34 +1545,28 @@ sub get_element_value_help {
 }
 
 sub generate_doc {
-    my ( $self, $top_class_name, $dir, $done ) = @_;
+    my ( $self, $top_class_name, $dir_str, $done ) = @_;
 
     $done //= {} ;
     my $res = $self->get_model_doc($top_class_name, $done);
 
-    if ( defined $dir and $dir ) {
+    if ( defined $dir_str and $dir_str ) {
         foreach my $class_name ( sort keys %$res ) {
-            my $file = $class_name;
-            $file =~ s!::!/!g;
-            my $pl_file  = $dir . "/$file.pl";
-            my $pod_file = $dir . "/$file.pod";
-            my $pod_dir  = $pod_file;
-            $pod_dir =~ s!/[^/]+$!!;
-            make_path( $pod_dir, { mode => 0755 } ) unless -d $pod_dir;
+            my $dir = path($dir_str);
+            $dir->mkpath() unless $dir->exists;
+            my $file_path = $class_name;
+            $file_path =~ s!::!/!g;
+            my $pl_file  = $dir->child("$file_path.pl");
+            $pl_file->parent->mkpath unless $pl_file->parent->exists;
+            my $pod_file = $dir->child("$file_path.pod");
 
-            # -M returns age of file, not date or epoch. hence greater is older
             my $old = '';
-            if ( -e $pod_file ) {
-                my $fh = IO::File->new( $pod_file, 'r' ) || die "Can't open $pod_file: $!";
-                $old = join( '', $fh->getlines );
-                $fh->close;
+            if ($pod_file->exists ) {
+                $old = $pod_file->slurp_utf8;
             }
             if ( $old ne $res->{$class_name} ) {
-                my $fh = IO::File->new( $pod_file, 'w' ) || die "Can't open $pod_file: $!";
-                $fh->binmode(":utf8");
-                $fh->print( $res->{$class_name} );
-                $fh->close;
-                print "Wrote documentation in $pod_file\n";
+                $pod_file->spew_utf8( $res->{$class_name} );
+                say "Wrote documentation in $pod_file";
             }
         }
     }
@@ -1678,7 +1708,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -2243,6 +2273,23 @@ C<Foo/Bar.pl>. You can choose to specify the file containing
 the model with C<model_file> parameter. This is mostly useful for
 tests.
 
+The C<instance> method can also retrieve an instance that has already
+been created:
+
+ my $inst = $model->instance( name => 'test1' );
+
+=head2 get_instance
+
+Retrieve an existing instance using its name.
+
+ my $inst = $model->get_instance('test1' );
+
+=head2 has_instance
+
+Check if an instance name already exists
+
+  my $maybe = $model->has_instance('test1');
+
 =head2 cme ( ... )
 
 This method is syntactic sugar for short program. It creates a new
@@ -2685,7 +2732,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
