@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::AnyId;
-$Config::Model::AnyId::VERSION = '2.096';
+$Config::Model::AnyId::VERSION = '2.098';
 use 5.010;
 
 use Mouse;
@@ -277,18 +277,35 @@ sub set_properties {
 
 sub create_default_with_init {
     my $self = shift;
+    my $idx = shift;
 
     return unless defined $self->{default_with_init};
 
     my $h = $self->{default_with_init};
     foreach my $def_key ( keys %$h ) {
-        my $v_obj = $self->fetch_with_id($def_key);
-        if ( $v_obj->get_type eq 'leaf' ) {
-            $v_obj->store( $h->{$def_key} );
-        }
-        else {
-            $v_obj->load( $h->{$def_key} );
-        }
+        $self->create_default_content($def_key);
+    }
+}
+
+sub create_default_content {
+    my $self = shift;
+    my $idx = shift //  die "missing index";
+
+    return unless defined $self->{default_with_init};
+
+    my $def = $self->{default_with_init}{$idx};
+    return unless defined $def; # no default content to create for $idx
+
+    return if $self->_defined($idx) ; # object already created
+
+    $self->auto_vivify($idx);
+
+    my $v_obj = $self->_fetch_with_id($idx);
+    if ( $v_obj->get_type eq 'leaf' ) {
+        $v_obj->store( $def );
+    }
+    else {
+        $v_obj->load( $def );
     }
 }
 
@@ -707,6 +724,9 @@ sub fetch_with_id {
         or $check eq 'no';
 
     if ( $ok or $check eq 'no' ) {
+        # create another method
+        $self->create_default_content($idx); # no-op if idx exists
+
         $self->auto_vivify($idx) unless $self->_defined($idx);
         return $self->_fetch_with_id($idx);
     }
@@ -1011,7 +1031,7 @@ Config::Model::AnyId - Base class for hash or list element
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -1182,13 +1202,17 @@ or C<< default_keys => ['foo', 'bar'] >>.
 
 To perform special set-up on children nodes you can also use 
 
-   default_with_init =>  { 'foo' => 'X=Av Y=Bv' ,
-                           'bar' => 'Y=Av Z=Cv' }
+   default_with_init =>  {
+      foo => 'X=Av Y=Bv',
+      bar => 'Y=Av Z=Cv'
+   }
 
 When the hash contains leaves, you can also use:
 
-   default_with_init => { 'def_1' => 'def_1 stuff' ,
-                          'def_2' => 'def_2 stuff' }
+   default_with_init => {
+       def_1 => 'def_1 stuff',
+       def_2 => 'def_2 stuff'
+   }
 
 =item migrate_keys_from
 
@@ -1595,7 +1619,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
