@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::Any;
-$Config::Model::Backend::Any::VERSION = '2.096';
+$Config::Model::Backend::Any::VERSION = '2.098';
 use Carp;
 use strict;
 use warnings;
@@ -30,6 +30,8 @@ has 'node' => (
     handles => ['show_message', 'instance'],
 );
 
+sub skip_open { return 0; }
+
 sub suffix {
     my $self = shift;
     $logger->info(
@@ -185,7 +187,7 @@ Config::Model::Backend::Any - Virtual class for other backends
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -359,8 +361,8 @@ the C<write> call-back with the file handle with C<io_handle>
 parameter. C<write> should use this handle to write data in the target
 configuration file.
 
-If this behavior causes problem, the solution is either to override
-C<skip_open> method in your backend that returns C<1>.
+If this behavior causes problem, the solution is to override
+C<skip_open> method in your backend to return C<1>.
 
 =back
 
@@ -507,7 +509,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
