@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::HashId;
-$Config::Model::HashId::VERSION = '2.096';
+$Config::Model::HashId::VERSION = '2.098';
 use Mouse;
 use 5.10.1;
 
@@ -27,6 +27,7 @@ my $logger = get_logger("Tree::Element::Id::Hash");
 extends qw/Config::Model::AnyId/;
 
 with "Config::Model::Role::Grab";
+with "Config::Model::Role::ComputeFunction";
 
 has data => ( is => 'rw', isa => 'HashRef',  default => sub { {}; } );
 has list => (
@@ -149,7 +150,6 @@ sub _fetch_all_indexes {
 # fetch without any check
 sub _fetch_with_id {
     my ( $self, $key ) = @_;
-    my $i = $self->instance;
     return $self->{data}{$key};
 }
 
@@ -538,7 +538,7 @@ Config::Model::HashId - Handle hash element for configuration model
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -656,7 +656,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
