@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Loader;
-$Config::Model::Loader::VERSION = '2.097';
+$Config::Model::Loader::VERSION = '2.098';
 use Carp;
 use strict;
 use warnings;
@@ -38,6 +38,8 @@ sub load {
     my $steps = delete $args{steps} // delete $args{step};
     croak "load error: missing 'steps' parameter" unless defined $steps;
 
+    my $caller_is_root = delete $args{caller_is_root};
+
     if (delete $args{experience}) {
         carp "load: experience parameter is deprecated";
     }
@@ -82,8 +84,10 @@ sub load {
 
         # found '!' command
         if ( $ret eq 'root' ) {
-            $logger->debug("Setting current_node to root node");
-            $current_node = $current_node->root;
+            $current_node = $caller_is_root ? $node : $current_node->root;
+            if ($logger->debug) {
+                $logger->debug("Setting current_node to root node: ".$current_node->name);
+            }
         }
     } while ( $ret eq 'root' );
 
@@ -708,16 +712,16 @@ sub _load_hash {
         $self->_load_value( $obj, $check, $subaction, $value )
             and return 'ok';
     }
-    elsif ( $action eq ':' and defined $note ) {
-
-        # action was just to store annotation
+    elsif ( $action eq ':' ) {
+        $logger->debug("_load_hash: created empty element of type $cargo_type");
         return 'ok';
     }
     elsif ($action) {
+        $logger->debug("_load_hash: giving up");
         Config::Model::Exception::Load->throw(
             object  => $element,
             command => join( '', grep { defined $_ } @$inst ),
-            error   => "Hash assignment with '$action' on unexpected " . "cargo_type: $cargo_type"
+            error   => "Hash load with '$action' on unexpected " . "cargo_type: $cargo_type"
         );
     }
 }
@@ -826,7 +830,7 @@ Config::Model::Loader - Load serialized data into config tree
 
 =head1 VERSION
 
-version 2.097
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -1209,6 +1213,11 @@ L<above/"load string syntax"> for a description of the string.
 Whether to check values while loading. Either C<yes> (default), C<no> or C<skip>.
 Bad values are discarded when C<check> is set to C<skip>.
 
+=item caller_is_root
+
+Change the target of the C<!> command: when set, the C<!> command go
+to caller node instead of going to root node. (default is false)
+
 =back
 
 =head1 AUTHOR
@@ -1225,7 +1234,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
