@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Node;
-$Config::Model::Node::VERSION = '2.097';
+$Config::Model::Node::VERSION = '2.098';
 use Mouse;
 with "Config::Model::Role::NodeLoader";
 
@@ -31,6 +31,7 @@ extends qw/Config::Model::AnyThing/;
 
 with "Config::Model::Role::Grab";
 with "Config::Model::Role::HelpAsText";
+with "Config::Model::Role::ComputeFunction";
 
 use vars qw(@status @level %default_property);
 
@@ -399,18 +400,6 @@ sub notify_change {
     }
 }
 
-sub write_back {
-    my ( $self, %args ) = @_;
-
-    my $force_write = delete $args{force} || 0;
-
-    if ( $self->location and $args{config_file} ) {
-        die "write_back: cannot override config_file in non root node (", $self->location, ")\n";
-    }
-
-    $self->backend_mgr->write_back(%args) if $self->needs_save or $force_write;
-}
-
 sub is_auto_write_for_type {
     my $self = shift;
     return 0 unless defined $self->backend_mgr;
@@ -1215,7 +1204,7 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.097
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -1621,9 +1610,9 @@ L<Config::Model::AnyThing>
 
 See L<Config::Model::AnyThing/"index_value()">
 
-=head2 parent()
+=head2 parent
 
-See L<Config::Model::AnyThing/"parent()">
+See L<Config::Model::AnyThing/"parent">
 
 =head2 root()
 
@@ -1926,7 +1915,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
