@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Role::Grab;
-$Config::Model::Role::Grab::VERSION = '2.096';
+$Config::Model::Role::Grab::VERSION = '2.098';
 # ABSTRACT: Role to grab data from elsewhere in the tree
 
 use Mouse::Role;
@@ -366,11 +366,14 @@ Config::Model::Role::Grab - Role to grab data from elsewhere in the tree
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
- $self->load_node( config_class_name => "...", %other_args);
+  $root->grab('foo:2 bar');
+  $root->grab(steps => 'foo:2 bar');
+  $root->grab(steps => 'foo:2 bar', type => 'leaf');
+  $root->grab_value(steps => 'foo:2 bar');
 
 =head1 DESCRIPTION
 
@@ -411,8 +414,8 @@ throws an exception or returns the last object found with the requested type.
 
 Examples:
 
- $root->grep(steps => 'foo:2 bar', type => 'leaf')
- $root->grep(steps => 'foo:2 bar', type => ['leaf','check_list'])
+ $root->grab(steps => 'foo:2 bar', type => 'leaf')
+ $root->grab(steps => 'foo:2 bar', type => ['leaf','check_list'])
 
 =item C<autoadd>
 
@@ -497,7 +500,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
