@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::TreeSearcher;
-$Config::Model::TreeSearcher::VERSION = '2.097';
+$Config::Model::TreeSearcher::VERSION = '2.098';
 use Mouse;
 use Mouse::Util::TypeConstraints;
 
@@ -104,7 +104,7 @@ sub search {
         foreach my $e (@element) {
             my $store = 0;
 
-            map { $store = 1 if $need_search->{$_} and $node->get_help( $_ => $e ) =~ $reg }
+            map { $store = 1 if $need_search->{$_} and $node->get_help_as_text( $_ => $e ) =~ $reg }
                 qw/description summary/;
             $store = 1 if $need_search->{element} and $e =~ $reg;
 
@@ -147,7 +147,7 @@ Config::Model::TreeSearcher - Search tree for match in value, description...
 
 =head1 VERSION
 
-version 2.097
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -249,7 +249,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
