@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::ValueComputer;
-$Config::Model::ValueComputer::VERSION = '2.096';
+$Config::Model::ValueComputer::VERSION = '2.098';
 use Mouse;
 use MouseX::StrictConstructor;
 
@@ -28,7 +28,13 @@ has formula    => ( is => 'ro', isa => 'Str', required => 1 );
 has value_type => ( is => 'ro', isa => 'Str', required => 1 );
 
 # value_object is mostly used for error messages
-has value_object => ( is => 'ro', isa => 'Config::Model::AnyThing', required => 1, weak_ref => 1 );
+has value_object => (
+    is => 'ro',
+    isa => 'Config::Model::AnyThing',
+    required => 1,
+    weak_ref => 1,
+    handles => [qw/grab grab_value location index element/],
+);
 
 has variables => ( is => 'ro', isa => 'HashRef', default => sub { {} } );
 has replace   => ( is => 'ro', isa => 'HashRef', default => sub { {} } );
@@ -89,8 +95,8 @@ sub BUILD {
         $compute_parser->pre_compute( $self->{formula}, 1, $self->{value_object},
         $self->{variables}, $self->{replace}, 'yes', $self->need_quote, );
 
-    $logger->debug("pre_formula: $$result_r");
-    $self->{pre_formula} = $$result_r;
+    $logger->debug("pre_formula: ". ($result_r ? $$result_r : ' pre_compute failed, using original formula'));
+    $self->{pre_formula} = $result_r ? $$result_r : $self->{formula};
 }
 
 sub compute {
@@ -314,45 +320,8 @@ sub _function_on_object {
 
     $logger->debug("handling &$function($up) ");
 
-    # get now the object refered
-    $up =~ s/-(\d+)/'- ' x $1/e;
-
-    my $target = eval { $value_object->grab( step => $up, check => $check ) };
-
-    if ($@) {
-        my $e = $@;
-        my $msg = $e ? $e->full_message : '';
-        Config::Model::Exception::Model->throw(
-            object => $value_object,
-            error  => "Compute function argument '$up':\n" . $msg
-        );
-    }
-
-    if ( $function eq 'element' ) {
-        my $result = $target->element_name;
-        Config::Model::Exception::Model->throw(
-            object => $value_object,
-            error  => "'",
-            $target->name, "' has no element name"
-        ) unless defined $result;
-        $return = \$result;
-    }
-    elsif ( $function eq 'index' ) {
-        my $result = $target->index_value;
-        Config::Model::Exception::Formula->throw(
-            object => $value_object,
-            error  => "'",
-            $target->name, "' has no index value"
-        ) unless defined $result;
-        $return = \$result;
-    }
-    else {
-        Config::Model::Exception::Formula->throw(
-            object => $value_object,
-            error  => "Unknown computation function &$function, "
-                . "expected &element(...) or &index(...)"
-        );
-    }
+    my $target = $value_object->eval_function($function, $up, $check);
+    $return = \$target ;
 
     # print "\&foo(...) result = ",$$return," \n";
 
@@ -499,7 +468,7 @@ pre_value:
   <skip:''> '$replace' '{' /\s*/ pre_value[@arg] /\s*/ '}' {
     $return = Config::Model::ValueComputer::_pre_replace($arg[2], ${ $item{pre_value} } ) ;
   }
-  | <skip:''> function '(' <commit> /\s*/ up /\s*/ ')' {
+  | <skip:''> function '(' /\s*/ up /\s*/ ')' {
     $return = Config::Model::ValueComputer::_function_on_object($item{up},$item{function},$return,@arg ) ;
   }
   | <skip:''> '&' /\w+/ func_param(?) {
@@ -523,7 +492,7 @@ pre_value:
 
 func_param: /\(\s*\)/
 
-up: /-?\d*/
+up: /-\d+|-( ?-)*/
 
 compute:  <skip:''> value[@arg](s) { 
     # if one value is undef, return undef;
@@ -581,7 +550,7 @@ Config::Model::ValueComputer - Provides configuration value computation
 
 =head1 VERSION
 
-version 2.096
+version 2.098
 
 =head1 SYNOPSIS
 
@@ -592,20 +561,21 @@ version 2.096
  $model ->create_config_class (
     name => "MyClass",
 
-    element => [ 
-
-       [qw/av bv/] => {type => 'leaf',
-                       value_type => 'integer',
-                      },
-       compute_int => { 
-	    type => 'leaf',
-            value_type => 'integer',
-            compute    => { formula   => '$a + $b', 
-                            variables => { a => '- av', b => '- bv'}
-                          },
-          },
-   ],
- ) ;
+    element => [
+       [qw/av bv/] => {
+           type => 'leaf',
+           value_type => 'integer',
+       },
+       compute_int => {
+	       type => 'leaf',
+               value_type => 'integer',
+               compute    => {
+                   formula   => '$a + $b',
+                   variables => { a => '- av', b => '- bv'}
+               },
+       },
+    ],
+ );
 
  my $inst = $model->instance(root_class_name => 'MyClass' );
 
@@ -688,7 +658,7 @@ The index value of the current object : C<&index> or C<&index()>.
 =item *
 
 The index value of a parent object: C<&index(-)>. Ancestor index value can be retrieved
-with C<&index(-2)> or C<&index(-3)>.
+with C<&index(-2)> or C<&index(-3)> or C<&index(- -)> or C<&index(- - -)>
 
 =item *
 
@@ -718,16 +688,18 @@ is used as the computed value.
 For instance:
 
   # extract host from url
-  compute => { formula => '$old =~ m!http://[\w\.]+(?::\d+)?(/.*)!; $1 ;', 
-	       variables => { old => '- url' } ,
-	       use_eval => 1 ,
-	     },
+  compute => {
+      formula => '$old =~ m!http://[\w\.]+(?::\d+)?(/.*)!; $1 ;',
+      variables => { old => '- url' } ,
+	  use_eval => 1 ,
+  },
 
   # capitalize
-  compute => { formula => 'uc($old)',
-	       variables => { old => '- small_caps' } ,
-	       use_eval => 1 
-             }
+  compute => {
+      formula => 'uc($old)',
+	  variables => { old => '- small_caps' } ,
+	  use_eval => 1
+  }
 
 =head2 Compute variables
 
@@ -739,33 +711,36 @@ the correct L<Value|Config::Model::Value> object.
 In this numeric example, C<result> default value is C<av + bv>:
 
  element => [
-  av => { 
-    type => 'leaf',
-    value_type => 'integer'
-  },
-  bv => { 
-    type => 'leaf',
-    value_type => 'integer'
-  },
-  result => { 
-    type => 'leaf',
-    value_type => 'integer', 
-    compute => { formula => '$a + $b' , 
-                 variables => { a => '- av', b => '- bv' },
-               }
-  }
+    av => {
+        type => 'leaf',
+        value_type => 'integer'
+    },
+    bv => {
+        type => 'leaf',
+        value_type => 'integer'
+    },
+    result => {
+        type => 'leaf',
+        value_type => 'integer',
+        compute => {
+            formula => '$a + $b' ,
+            variables => { a => '- av', b => '- bv' },
+        }
+    }
+ ]
 
 In this string example, the default value of the C<Comp> element is
 actually a string made of "C<macro is >" and the value of the
 "C<macro>" element of the object located 2 nodes above:
 
-   comp => { 
+ comp => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' ,
-                 variables => { m => '- - macro' }
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' }
+    }
+ }
 
 =head2 Compute replace
 
@@ -776,17 +751,18 @@ C<%replace> hash.
 
 For instance, if you want to display a summary of a config, you can do :
 
-       compute_with_replace 
-       => {
-            formula => '$replace{$who} is the $replace{$what} of $replace{$country}',
-            variables => {
-                           who   => '! who' ,
-                           what  => '! what' ,
-                           country => '- country',
-                         },
-            replace => {  chief => 'president', 
-                          America => 'USA'
-                       },
+ compute_with_replace => {
+     formula => '$replace{$who} is the $replace{$what} of $replace{$country}',
+     variables => {
+         who   => '! who' ,
+         what  => '! what' ,
+         country => '- country',
+     },
+     replace => {
+         chief => 'president',
+         America => 'USA'
+     }
+ }
 
 =head2 Complex formula
 
@@ -801,19 +777,23 @@ I.e. C<&element($foo)> is ok, but C<&element(&index($foo))> is not allowed.
 
 Compute variables can themselves be computed :
 
-   compute => {
-     formula => 'get_element is $replace{$s}, indirect value is \'$v\'',
-     variables => { 's' => '! $where',
-                     where => '! where_is_element',
-                     v => '! $replace{$s}',
-                  }
-     replace   => { m_value_element => 'm_value',
-                    compute_element => 'compute' 
-                  }
+ compute => {
+    formula => 'get_element is $replace{$s}, indirect value is \'$v\'',
+    variables => {
+        s => '! $where',
+        where => '! where_is_element',
+        v => '! $replace{$s}',
     }
+    replace => {
+        m_value_element => 'm_value',
+        compute_element => 'compute'
+    }
+ }
 
 Be sure not to specify a loop when doing recursive computation.
 
+The function C<&index> C<&element> and C<&location> are also allowed.
+
 =head2 compute override
 
 In some case, a computed value must be interpreted as a default value
@@ -821,30 +801,32 @@ and the user must be able to override this computed default value.  In
 this case, you must use C<< allow_override => 1 >> with the
 compute parameter:
 
-   computed_value_with_override => { 
+ computed_value_with_override => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' , 
-                 variables => { m => '- - macro' } ,
-                 allow_override => 1,
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' } ,
+        allow_override => 1,
+    }
+ }
 
 This computed default value is written to the configuration file.
 
 This default value may be already known by the application so the computed value
-should B<not> be written to the configuration file. The computed value is interesting 
+should B<not> be written to the configuration file. The computed value is interesting
 because it cab be shown to the user. In this case, use the C<use_as_upstream_default>
-parameter: 
+parameter:
 
-   compute_known_upstream => { 
+ compute_known_upstream => {
     type => 'leaf',
-    value_type => 'string', 
-    compute => { formula => '"macro is $m"' , 
-                 variables => { m => '- - macro' } ,
-                 use_as_upstream_default => 1,
-               }
-   }
+    value_type => 'string',
+    compute => {
+        formula => '"macro is $m"' ,
+        variables => { m => '- - macro' } ,
+        use_as_upstream_default => 1,
+    }
+ }
 
 C<use_as_upstream_default> implies C<allow_override>.
 
@@ -865,80 +847,80 @@ The most useful fallback values are:
 
 Example:
 
-        Source => {
-            value_type   => 'string',
-            mandatory    => 1,
-            migrate_from => {
-                use_eval  => 1,
-                formula   => '$old || $older ;',
-                undef_is => "''",
-                variables => {
-                    older => '- Original-Source-Location',
-                    old   => '- Upstream-Source'
-                }
-            },
-            type => 'leaf',
-        },
-        [qw/Upstream-Source Original-Source-Location/] => {
-            value_type => 'string',
-            status     => 'deprecated',
-            type       => 'leaf'
+ Source => {
+    type => 'leaf',
+    value_type   => 'string',
+    mandatory    => 1,
+    migrate_from => {
+        use_eval  => 1,
+        formula   => '$old || $older ;',
+        undef_is => "''",
+        variables => {
+            older => '- Original-Source-Location',
+            old   => '- Upstream-Source'
         }
+    },
+ },
+ [qw/Upstream-Source Original-Source-Location/] => {
+    value_type => 'string',
+    status     => 'deprecated',
+    type       => 'leaf'
+ }
 
 =head1 Examples
 
 =head2 String substitution
 
-    [qw/sav sbv/] => {
-        type       => 'leaf',
-        value_type => 'string',
-      },
-    compute_string => {
-        type       => 'leaf',
-        value_type => 'string',
-        compute    => {
-            formula   => 'meet $a and $b',
-            variables => { '- sav', b => '- sbv' }
-        },
-    },
+ [qw/sav sbv/] => {
+     type       => 'leaf',
+     value_type => 'string',
+   },
+ compute_string => {
+     type       => 'leaf',
+     value_type => 'string',
+     compute    => {
+         formula   => 'meet $a and $b',
+         variables => { '- sav', b => '- sbv' }
+     },
+ },
 
 =head2 Computation with on-the-fly replacement
 
-    compute_with_replace => {
-        type       => 'leaf',
-        value_type => 'string',
-        compute    => {
-            formula =>
-              '$replace{$who} is the $replace{$what} of $replace{$country}',
-            variables => {
-                who     => '! who',
-                what    => '! what',
-                country => '- country',
-            },
-            replace => {
-                chief   => 'president',
-                America => 'USA'
-            },
-        },
-      },
+ compute_with_replace => {
+     type       => 'leaf',
+     value_type => 'string',
+     compute    => {
+         formula =>
+           '$replace{$who} is the $replace{$what} of $replace{$country}',
+         variables => {
+             who     => '! who',
+             what    => '! what',
+             country => '- country',
+         },
+         replace => {
+             chief   => 'president',
+             America => 'USA'
+         },
+     },
+   },
 
 =head2 Extract data from a value using a Perl regexp
 
 Extract the host name from an URL:
 
-    url => {
-        type       => 'leaf',
-        value_type => 'uniline'
-    },
-    extract_host_from_url => {
-        type       => 'leaf',
-        value_type => 'uniline',
-        compute    => {
-            formula   => '$old =~ m!http://([\w\.]+)!; $1 ;',
-            variables => { old => '- url' },
-            use_eval  => 1,
-        },
-    },
+ url => {
+     type       => 'leaf',
+     value_type => 'uniline'
+ },
+ extract_host_from_url => {
+     type       => 'leaf',
+     value_type => 'uniline',
+     compute    => {
+         formula   => '$old =~ m!http://([\w\.]+)!; $1 ;',
+         variables => { old => '- url' },
+         use_eval  => 1,
+     },
+ },
 
 =head2 simple copy hash example
 
@@ -1033,7 +1015,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2016 by Dominique Dumont.
+This software is Copyright (c) 2005-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
