@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,26 +9,26 @@
 #
 [
   {
-    'name' => 'Fstab::Ext4FsOpt',
-    'include' => [
-      'Fstab::Ext2FsOpt'
+    'author' => [
+      'Dominique Dumont'
     ],
     'copyright' => [
       '2010,2011 Dominique Dumont'
     ],
-    'author' => [
-      'Dominique Dumont'
-    ],
-    'license' => 'LGPL2',
     'element' => [
       'lazy_itable_init',
       {
-        'value_type' => 'boolean',
-        'upstream_default' => '1',
+        'description' => 'If enabled and the uninit_bg feature is enabled, the inode table will not be fully initialized by mke2fs. This speeds up filesystem initialization noticeably, but it requires the kernel to finish initializing the filesystem in the background when the filesystem is first mounted.',
         'type' => 'leaf',
-        'description' => "If enabled and the uninit_bg feature is enabled, the inode table will not be fully initialized by mke2fs. This speeds up filesystem initialization noticeably, but it requires the kernel to finish initializing the filesystem in the background when the filesystem is first mounted."
+        'upstream_default' => '1',
+        'value_type' => 'boolean'
       }
-    ]
+    ],
+    'include' => [
+      'Fstab::Ext2FsOpt'
+    ],
+    'license' => 'LGPL2',
+    'name' => 'Fstab::Ext4FsOpt'
   }
 ]
 ;
