@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2016 by Dominique Dumont.
+# This software is Copyright (c) 2005-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -10,10 +10,9 @@
 
 $model_to_test = "Multistrap";
 
-$from_scratch_file = <<'EOF' ,
+$from_scratch_file = <<'EOF' ;
 ## This file was written by cme command.
-## You can run 'cme edit <application>' to modify this file.
-## Run 'cme list' to get the list of applications available on your system
+## You can run 'cme edit multistrap' to modify this file.
 ## You may also modify the content of this file with your favorite editor.
 
 [general]
